
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_HIT_RATIOS_STATS_H
#define PG_HIT_RATIOS_STATS_H

#include "base/TmSzStat.h"

// handy for keeping track of hit ratios, cachability ratios, etc.
// "hit" and "miss" classes must be disjoint
class HRStat {
	public:
		HRStat();

		void reset();
		OLog &store(OLog &log) const;
		ILog &load(ILog &);

		inline void record(const Time &tm, Size sz, bool hit);

		const TmSzStat &hits() const { return theHits; }
		const TmSzStat &misses() const { return theMisses; }
		TmSzStat xacts() const;

		double dhr() const;
		double bhr() const;
		double dhp() const;
		double bhp() const;

		HRStat &operator +=(const HRStat &s);

		ostream &print(ostream &os, const String &hit, const String &miss, const String &pfx) const;

	protected:
		TmSzStat theHits;
		TmSzStat theMisses;
};

inline OLog &operator <<(OLog &ol, const HRStat &s) { return s.store(ol); }
inline ILog &operator >>(ILog &il, HRStat &s) { return s.load(il); }


/* inlined methods */

inline
void HRStat::record(const Time &tm, Size sz, bool hit) {
	hit ? theHits.record(tm, sz) : theMisses.record(tm, sz);
}

#endif
