
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef POLYGRAPH_OPTS_CLASSES_H
#define POLYGRAPH_OPTS_CLASSES_H

#include "base/opts.h"

class FileScanner;

// a collection of command line options that are very specific to Poly
// see include/opts.h for more generic options


// which system call to use for scanning for ready files
class FileScanOpt: public Opt {
	public:
		FileScanOpt(OptGrp *aGrp, const char *aName, const char *aDescr, FileScanner *def);

		virtual bool parse(const char *name, const char *val);
		virtual void report(ostream &) const;

		FileScanner *val() { return theVal; }
		operator void*() const { return theVal ? (void*)-1 : 0; }

	protected:
		FileScanner *theVal;
};


// various message dump flags
typedef enum { dumpNone = 0, dumpAny = ~0U,
	dumpReq = 1, dumpRep = 2, dumpErr = 3, dumpTypeCnt,
	dumpHdr = 1, dumpBody = 2 } DumpFlags;

class DumpFlagsOpt: public ListOpt {
	public:
		DumpFlagsOpt(OptGrp *aGrp, const char *aName, const char *aDescr);

		virtual void report(ostream &) const;

		bool operator ()(DumpFlags type, DumpFlags part) const { return (theFlags[type] & part) != 0; }

		void setAll(); // dump everything

	protected:
		virtual bool addItem(const String &item);
		void setFlag(DumpFlags type, DumpFlags part);
		const char *dumpPartStr(DumpFlags type) const;

	protected:
		unsigned int theFlags[dumpTypeCnt];
};


// generic interface, but poly-specific implementation
class HostTypeOpt: public Opt {
	public:
		HostTypeOpt(OptGrp *aGrp, const char *aName, const char *aDescr);

		virtual bool parse(const char *name, const char *val);
		virtual void report(ostream &) const;
};

// generic interface, but poly-specific implementation
class VersionOpt: public Opt {
	public:
		VersionOpt(OptGrp *aGrp, const char *aName, const char *aDescr);

		virtual bool parse(const char *name, const char *val);
		virtual void report(ostream &) const;
};

#endif
