
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_PRIVATE_CACHE_H
#define PG_PRIVATE_CACHE_H

#include "xstd/Time.h"
#include "xstd/Array.h"
#include "xstd/Ring.h"

class ObjId;

class PrivCacheEntry {
	public:
		operator void *() const { return theName > 0 ? (void*)-1 : 0; }

	public:
		int theName;      // object name
		int theWorld;     // object wid hash
		int theStampSecs; // timestamp [in seconds]		
};

// an LRU-driven, slightly imprecise, memory inefficient cache
class PrivCache {
	public:
		PrivCache(int aCapacity = 0);

		void capacity(int aCap);

		bool hasOid(const ObjId &oid) const { return stamp(oid) > 0; }
		Time stamp(const ObjId &oid) const;

		void putOid(const ObjId &oid, Time stamp);

	protected:
		int hash(const ObjId &oid) const;
		void purgeOne();

	protected:
		typedef unsigned short IdxItem;
		Ring<IdxItem> theHistory;       // index into a hash
		Array<PrivCacheEntry> theHash;
};

#endif
