
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "xstd/BigSize.h"
#include "pgl/PglRec.h"
#include "pgl/PglIntSym.h"
#include "pgl/PglNumSym.h"
#include "pgl/PglTimeSym.h"
#include "pgl/PglSizeSym.h"
#include "pgl/GoalSym.h"


GoalSym::GoalSym(): RecSym("Goal", new PglRec) {

	// add fields

	theRec->bAdd("time", "duration", 0);
	theRec->bAdd("int", "xactions", 0);
	theRec->bAdd("size", "fill_size", 0);
	theRec->bAdd("float", "errors", 0);
}

GoalSym::GoalSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool GoalSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Goal";
}

SynSym *GoalSym::dupe(const String &type) const {
	if (isA(type))
		return new GoalSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}

Time GoalSym::duration() const {
	return getTime("duration");
}

bool GoalSym::fillSize(BigSize &sz) const {
	SynSymTblItem *szi = 0;
	Assert(theRec);
	Assert(theRec->find("fill_size", szi));
	if (szi->sym())
		sz = ((const SizeSym&)szi->sym()->cast("size")).val();
	return szi->sym() != 0;
}

bool GoalSym::xactCount(int &count) const {
	SynSymTblItem *xi = 0;
	Assert(theRec);
	Assert(theRec->find("xactions", xi));
	if (xi->sym())
		count = ((const IntSym&)xi->sym()->cast("int")).val();
	return xi->sym() != 0;
}

bool GoalSym::errs(double &ratio, int &count) const {
	SynSymTblItem *ei = 0;
	Assert(theRec);
	Assert(theRec->find("errors", ei));
	if (ei->sym()) {
		double level = ((const NumSym&)ei->sym()->cast("float")).val();
		if (level < 1)
			ratio = level;
		else
			count = (int)level;
	}
	return ei->sym() != 0;
}

