
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef XPARSER_SYNTAX_SYMBOL_H
#define XPARSER_SYNTAX_SYMBOL_H

#include "xparser/LexToken.h"

// The SynSym class is the base class for all attributes
// stack can be found on the generated parser's stack.

class SynSym {
	public:
		SynSym(const String &aType);
		virtual ~SynSym();

		const String &type() const { return theType; }
		virtual bool isA(const String &type) const;
		bool canBe(const String &type) const;
		bool equal(const SynSym &s) const;

		// aborts if incompatible types
		const SynSym &cast(const String &type) const;
		SynSym &cast(const String &type);

		// creates a copy of itself or compatible type
		// returns 0 if incompatible types and by default
		SynSym *clone(const String &type) const;
		SynSym *clone() const { return clone(type()); }

		const TokenLoc &loc() const { return theLoc; }
		void loc(const TokenLoc &aLoc) { theLoc = aLoc; }

		virtual ostream &print(ostream &os, const String &pfx = 0) const;

	protected:
		void type(const String &aType);
		virtual SynSym *dupe(const String &) const { return 0; }

	protected:
		TokenLoc theLoc;

	private:
		String theType;
};

inline ostream &operator <<(ostream &os, const SynSym &sym) { return sym.print(os); }

#endif
