
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef XSTD_NET_IF_META_H
#define XSTD_NET_IF_META_H

#include "xstd/h/sys/socket.h"
#include "xstd/h/netinet.h"
#include "xstd/String.h"

#ifdef HAVE_NET_IF_H
#include <net/if.h>
#endif

#if defined(HAVE_WINSOCK2_H)
#include <winsock2.h>
#include <ws2tcpip.h>
#endif


#ifndef HAVE_TYPE_IFREQ
#   ifndef IFNAMSIZ
#   define IFNAMSIZ 16
#   endif
	struct ifreq {
		char ifr_name[IFNAMSIZ];
		sockaddr ifr_addr;
	};
#endif


class InetIfReq: public ifreq {
	typedef char Name[IFNAMSIZ];
	public:
		InetIfReq() { ifr_name[0] = (char)0; }
		InetIfReq(const ifreq &r): ifreq(r) {}

		const Name &name() const { return ifr_name; }
		const struct sockaddr_in &addr() const { return (sockaddr_in&)ifr_addr; }
		const struct in_addr &addrN() const { return ((sockaddr_in&)ifr_addr).sin_addr; }
};


#ifdef HAVE_TYPE_IFALIASREQ
	class InetIfAliasReq: public ifaliasreq {
		typedef char Name[IFNAMSIZ];

		public:
			inline InetIfAliasReq(const String &aName);
			InetIfAliasReq(const ifaliasreq &r): ifaliasreq(r) {}

			const Name &name() const { return ifra_name; }
			const struct sockaddr_in &addr() const { return (sockaddr_in&)ifra_addr; }
			const struct sockaddr_in &broad() const { return (sockaddr_in&)ifra_broadaddr; }
			const struct sockaddr_in &mask() const { return (sockaddr_in&)ifra_mask; }

			Name &name() { return ifra_name; }
			struct sockaddr_in &addr() { return (sockaddr_in&)ifra_addr; }
			struct sockaddr_in &broad() { return (sockaddr_in&)ifra_broadaddr; }
			struct sockaddr_in &mask() { return (sockaddr_in&)ifra_mask; }
	};
#else
#	ifndef IFNAMSIZ
#	define IFNAMSIZ 16
#	endif
	class InetIfAliasReq {
		typedef char Name[IFNAMSIZ];
		public:
			inline InetIfAliasReq(const String &aName);

			const Name &name() const { return ifra_name; }
			const struct sockaddr_in &addr() const { return (sockaddr_in&)ifra_addr; }
			const struct sockaddr_in &broad() const { return (sockaddr_in&)ifra_broadaddr; }
			const struct sockaddr_in &mask() const { return (sockaddr_in&)ifra_mask; }

			Name &name() { return ifra_name; }
			sockaddr_in &addr() { return (sockaddr_in&)ifra_addr; }
			sockaddr_in &broad() { return (sockaddr_in&)ifra_broadaddr; }
			sockaddr_in &mask() { return (sockaddr_in&)ifra_mask; }

		protected:
			char ifra_name[IFNAMSIZ];        // ifname, e.g. "en0"
			sockaddr_in ifra_addr;
			sockaddr_in ifra_broadaddr;
			sockaddr_in ifra_mask;  
	};
#endif


inline
InetIfAliasReq::InetIfAliasReq(const String &aName) {
	memset(this, 0, sizeof(*this));
	strncpy(name(), aName, sizeof(Name));
	addr().sin_family = AF_INET;
#ifdef HAVE_SA_LEN
	addr().sin_len = sizeof(sockaddr_in);
#endif
}


#endif
